const [INDETERMINATE, CHECKED, UNCHECKED] = ['INDETERMINATE', 'CHECKED', 'UNCHECKED'];
export class AbstractTreeNode {
    constructor(origin) {
        Object.defineProperty(this, "origin", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: origin
        });
        Object.defineProperty(this, "_parent", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: null
        });
    }
    get parent() {
        return this._parent;
    }
    get root() {
        let node = this;
        while (node.parent) {
            node = node.parent;
        }
        return node;
    }
    get isLeaf() {
        return !this.children;
    }
    get id() {
        return this._id;
    }
    get status() {
        return this._status;
    }
    get checked() {
        return this._status === CHECKED;
    }
    get unchecked() {
        return this._status === UNCHECKED;
    }
    get indeterminate() {
        return this._status === INDETERMINATE;
    }
    get disabled() {
        return this._disabled;
    }
    get enabled() {
        return !this._disabled;
    }
    updateStatus(checked, onlySelf = false) {
        if (this.children && !onlySelf) {
            this.children.forEach((child) => {
                child.updateStatus(checked, onlySelf);
            });
        }
        this._calculateStatus(checked);
    }
    _setParent(parent) {
        this._parent = parent;
    }
    _updateAncestors(checked, onlySelf) {
        if (this._parent && !onlySelf) {
            this._parent.updateStatus(checked, onlySelf);
        }
    }
}
export class SingleTreeNode extends AbstractTreeNode {
    constructor(origin, getId, opts) {
        super(origin);
        Object.defineProperty(this, "getId", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: getId
        });
        Object.defineProperty(this, "opts", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: opts
        });
        Object.defineProperty(this, "children", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_id", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_status", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_disabled", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this._id = getId(origin);
        this._disabled = !!(opts.disabled || origin.disabled);
        this._setUpChildren();
    }
    _setUpChildren() {
        if (this.origin.children) {
            this.children = this.origin.children.map((child) => {
                const node = new SingleTreeNode(child, this.getId, Object.assign(this.opts, {
                    disabled: this._disabled,
                }));
                node._setParent(this);
                return node;
            });
        }
    }
    _calculateStatus(checked) {
        this._status = this.id === checked ? CHECKED : UNCHECKED;
    }
}
export class MultipleTreeNode extends AbstractTreeNode {
    constructor(origin, getId, opts) {
        super(origin);
        Object.defineProperty(this, "getId", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: getId
        });
        Object.defineProperty(this, "opts", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: opts
        });
        Object.defineProperty(this, "children", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_id", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_status", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_disabled", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this._id = getId(origin);
        this._disabled = !!(opts.disabled || origin.disabled);
        this._setUpChildren();
    }
    changeStatus(status, checkeds) {
        const newCheckeds = this._changeStatus(status, checkeds);
        return newCheckeds;
    }
    _calculateStatus(checkeds) {
        if (this.children) {
            let checkedNum = 0;
            let hasIndeterminate = false;
            let enabledLength = 0;
            for (const child of this.children) {
                if (child.enabled) {
                    enabledLength += 1;
                }
                if (child.checked) {
                    checkedNum += 1;
                }
                else if (child.enabled && child.indeterminate) {
                    hasIndeterminate = true;
                }
            }
            this._status = hasIndeterminate
                ? INDETERMINATE
                : checkedNum === 0
                    ? UNCHECKED
                    : checkedNum === enabledLength
                        ? CHECKED
                        : INDETERMINATE;
        }
        else {
            this._status = checkeds.has(this._id) ? CHECKED : UNCHECKED;
        }
    }
    _changeStatus(status, initValue) {
        let res = new Set(initValue);
        if (this.enabled && this._status !== status) {
            this._status = status;
            if (this.children) {
                this.children.forEach((child) => {
                    res = child._changeStatus(status, res);
                });
            }
            else {
                if (status === CHECKED) {
                    res.add(this._id);
                }
                else {
                    res.delete(this._id);
                }
            }
        }
        return res;
    }
    _setUpChildren() {
        if (this.origin.children) {
            this.children = this.origin.children.map((v) => {
                const node = new MultipleTreeNode(v, this.getId, Object.assign(this.opts, {
                    disabled: this._disabled,
                }));
                node._setParent(this);
                return node;
            });
        }
    }
}
//# sourceMappingURL=node.js.map